import Flutter
import UIKit

public class BradyFlutterPlugin: NSObject, FlutterPlugin {
    
  var bradySdk: BradySdkApi
    
  init(bradySdk: BradySdkApi) {
      self.bradySdk = bradySdk
  }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "brady_flutter_plugin", binaryMessenger: registrar.messenger())
    let instance = BradyFlutterPlugin(bradySdk: BradySdkApi())
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "getPlatformVersion":
        result("iOS " + UIDevice.current.systemVersion)
    case "startPrinterDiscovery":
        bradySdk.startPrinterDiscovery()
    case "getPrinters":
        result(bradySdk.getPrinters())
    case "connectToPrinter":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        let printerName = myArgs?["printerName"] as? String
        Task {
            let connected = await bradySdk.connectToPrinter(printer: printerName)
            if connected {
                bradySdk.stopPrinterDiscovery()
            }
            result(connected)
        }
    case "getPrinterDetails":
        let details = bradySdk.getPrinterDetails()
        result(bradySdk.getPrinterDetails())
    case "getLastConnectedPrinterName":
        let printerName = bradySdk.getLastConnectedPrinterName()
        result(printerName)
    case "setTemplate":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        guard let dataString = myArgs?["byteData"] as? String else { return }
        guard let data = Data(base64Encoded: dataString) else { return }
        let iStream = InputStream(data: data)
        bradySdk.setTemplate(inputStream: iStream)
    case "setPlaceholderValue":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        guard let placeholder = myArgs?["placeholder"] as? String else { return }
        guard let value = myArgs?["value"] as? String else { return }
        bradySdk.setPlaceholderValue(placeholder: placeholder, value: value)
    case "print":
        Task {
            let printed = await bradySdk.print()
            result(printed)
        }
    case "feed":
        Task {
            let fed = await bradySdk.feed()
            result(fed)
        }
    case "cut":
        Task {
            let cut = await bradySdk.cut()
            result(cut)
        }
    case "disconnect":
        Task {
            let disconnected = await bradySdk.disconnect()
            result(disconnected)
        }
    default:
      result(FlutterMethodNotImplemented)
    }
  }
}
