import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'brady_flutter_plugin_platform_interface.dart';

/// An implementation of [BradyFlutterPluginPlatform] that uses method channels.
class MethodChannelBradyFlutterPlugin extends BradyFlutterPluginPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('brady_flutter_plugin');

  @override
  Future<String?> getPlatformVersion() async {
    final version = await methodChannel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  @override
  void startPrinterDiscovery() {
    methodChannel.invokeMethod<String>('startPrinterDiscovery');
  }
  
  @override
  Future<List<String?>> getPrinters() async {
    String? printerNames = await methodChannel.invokeMethod<String>('getPrinters');
    List<String> printerNamesList = printerNames!.split(', ');
    printerNamesList.removeLast();
    return printerNamesList;
  }
  
  @override
  Future<bool?> connectToPrinter(String? printer) async {
    var sendMap = <String, dynamic> {
      "printerName" : printer
    };
    bool? connected = await methodChannel.invokeMethod<bool>('connectToPrinter', sendMap);
    return connected;
  }

  @override
  Future<String?> getPrinterDetails() async {
    String? details = await methodChannel.invokeMethod<String>('getPrinterDetails');
    return details;
  }

  @override
  Future<String?> getLastConnectedPrinterName() async {
    String? lastConnectedPrinterName = await methodChannel.invokeMethod<String>('getLastConnectedPrinterName');
    return lastConnectedPrinterName;
  }

  @override
  void setTemplate(String dataString) {
    var sendMap = <String, dynamic> {
      "byteData" : dataString
    };
    methodChannel.invokeMethod('setTemplate', sendMap);
  }

  @override
  void setPlaceholderValue(String placeholder, String value) {
    var sendMap = <String, dynamic> {
      "placeholder" : placeholder,
      "value" : value
    };
    methodChannel.invokeMethod('setPlaceholderValue', sendMap);
  }

  @override
  Future<bool?> print() async {
    bool? printed = await methodChannel.invokeMethod<bool>('print');
    return printed;
  }

  @override
  Future<bool?> feed() async {
    bool? fed = await methodChannel.invokeMethod<bool>('feed');
    return fed;
  }

  @override
  Future<bool?> cut() async {
    bool? cut = await methodChannel.invokeMethod<bool>('cut');
    return cut;
  }

  @override
  Future<bool?> disconnect() async {
    bool? disconnected = await methodChannel.invokeMethod<bool>('disconnect');
    return disconnected;
  }
}
