import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'brady_flutter_plugin_method_channel.dart';

abstract class BradyFlutterPluginPlatform extends PlatformInterface {
  /// Constructs a BradyFlutterPluginPlatform.
  BradyFlutterPluginPlatform() : super(token: _token);

  static final Object _token = Object();

  static BradyFlutterPluginPlatform _instance = MethodChannelBradyFlutterPlugin();

  /// The default instance of [BradyFlutterPluginPlatform] to use.
  ///
  /// Defaults to [MethodChannelBradyFlutterPlugin].
  static BradyFlutterPluginPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [BradyFlutterPluginPlatform] when
  /// they register themselves.
  static set instance(BradyFlutterPluginPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<String?> getPlatformVersion() {
    throw UnimplementedError('platformVersion() has not been implemented.');
  }

  void startPrinterDiscovery() {
    throw UnimplementedError('platformVersion() has not been implemented.');
  }
  
  Future<List<String?>> getPrinters() {
    throw UnimplementedError('getPrinters() has not been implemented.');
  }
  
  Future<bool?> connectToPrinter(String? printer) {
    throw UnimplementedError('connectToPrinter(String printer) has not been implemented.');
  }

  Future<String?> getPrinterDetails() {
    throw UnimplementedError('getPrinterDetails() has not been implemented.');
  }

  Future<String?> getLastConnectedPrinterName() {
    throw UnimplementedError('getLastConnectedPrinterName() has not been implemented.');
  }

  void setTemplate(String dataString) {
    throw UnimplementedError('setTemplate(String byteData) has not been implemented.');
  }

  void setPlaceholderValue(String placeholder, String value) {
    throw UnimplementedError('setTemplate(String placeholder, String value) has not been implemented.');
  }

  Future<bool?> print() {
    throw UnimplementedError('print() has not been implemented.');
  }

  Future<bool?> feed() {
    throw UnimplementedError('feed() has not been implemented.');
  }

  Future<bool?> cut() {
    throw UnimplementedError('cut() has not been implemented.');
  }

  Future<bool?> disconnect() {
    throw UnimplementedError('disconnect() has not been implemented.');
  }
}
